@@ -25,15 +25,19 @@
 import logging
 import traceback
 import calendar
-from impacket.dcerpc.v5 import transport, samr, srvs, lsat, lsad, nrpc, wkst, scmr, tsch
+import time
+import re
+from impacket.dcerpc.v5 import transport, samr, srvs, lsat, lsad, nrpc, wkst, scmr, tsch, rrp
 from impacket.dcerpc.v5.rpcrt import DCERPCException, RPC_C_AUTHN_LEVEL_PKT_INTEGRITY, RPC_C_AUTHN_LEVEL_PKT_PRIVACY
 from impacket.dcerpc.v5.ndr import NULL
 from impacket.dcerpc.v5.dtypes import RPC_SID, MAXIMUM_ALLOWED
 from impacket import smb3structs
 from bloodhound.ad.utils import ADUtils, AceResolver
 from bloodhound.enumeration.acls import parse_binary_acl
+from bloodhound.ad.structures import LDAP_SID
 from impacket.smb3 import SMB3
 from impacket.smb import SMB
+from impacket.smbconnection import SessionError
 # Try to import exceptions here, if this does not succeed, then impacket version is too old
 try:
     HostnameValidationExceptions = (SMB3.HostnameValidationException, SMB.HostnameValidationException)
@@ -58,6 +62,7 @@ class ADComputer(object):
         self.services = []
         self.sessions = []
         self.loggedon = []
+        self.registry_sessions = []
         self.addr = None
         self.smbconnection = None
         # The SID of the local domain
@@ -117,22 +122,23 @@ class ADComputer(object):
                 'FailureReason': None,
                 'Results': self.loggedon
             },
-            # Unsupported for now
             'RegistrySessions': {
-                'Collected': False,
+                'Collected': 'loggedon' in collect and not self.permanentfailure,
                 'FailureReason': None,
-                'Results': []
+                'Results': self.registry_sessions
             },
             'Aces': [],
             'HasSIDHistory': [],
             'IsDeleted': ADUtils.get_entry_property(entry, 'isDeleted', default=False),
             'Status': None
         }
+
         props = data['Properties']
         # via the TRUSTED_FOR_DELEGATION (0x00080000) flag in UAC
         props['unconstraineddelegation'] = ADUtils.get_entry_property(entry, 'userAccountControl', default=0) & 0x00080000 == 0x00080000
         props['enabled'] = ADUtils.get_entry_property(entry, 'userAccountControl', default=0) & 2 == 0
         props['trustedtoauth'] = ADUtils.get_entry_property(entry, 'userAccountControl', default=0) & 0x01000000 == 0x01000000
+        props['samaccountname'] = ADUtils.get_entry_property(entry, 'sAMAccountName')
 
         if 'objectprops' in collect or 'acl' in collect:
             props['haslaps'] = ADUtils.get_entry_property(entry, 'ms-mcs-admpwdexpirationtime', 0) != 0
@@ -286,6 +292,14 @@ class ADComputer(object):
                 logging.info('Ignoring host %s since its hostname does not match: %s', self.hostname, str(exc))
                 self.permanentfailure = True
                 return None
+            except SessionError as exc:
+                if ('STATUS_PIPE_NOT_AVAILABLE' in str(exc) or 'STATUS_OBJECT_NAME_NOT_FOUND' in str(exc)) and 'winreg' in binding.lower():
+                    # This can happen, silently ignore
+                    return None
+                # Else, just log it
+                logging.debug(traceback.format_exc())
+                logging.warning('DCE/RPC connection failed: %s', str(exc))
+                return None
 
             if self.smbconnection is None:
                 self.smbconnection = self.rpc.get_smb_connection()
@@ -421,6 +435,72 @@ class ADComputer(object):
 
         return sessions
 
+    def rpc_get_registry_sessions(self):
+        binding = r'ncacn_np:%s[\pipe\winreg]' % self.addr
+
+        # Try to bind to the Remote Registry RPC interface, if it fails try again once.
+        binding_attempts = 2
+        while binding_attempts > 0:
+            dce = self.dce_rpc_connect(binding, rrp.MSRPC_UUID_RRP)
+            if dce is None:
+                # If the Remote Registry is not yet started, the named pipe '\pipe\winreg' does not
+                # exist and therefore the following exception is expected: STATUS_PIPE_NOT_AVAILABLE.
+                # But this initial attempt should trigger it. Wait 1s and hope the service had enough
+                # time to start.
+                time.sleep(1)
+            else:
+                # We could connect to the Remote Registry, so exit the loop.
+                break
+            binding_attempts -= 1
+
+        # If the two binding attempts failed, silently return.
+        if dce is None:
+            logging.debug('Failed opening remote registry after 2 attempts')
+            return
+
+        registry_sessions = []
+
+        # Impacket's 'hOpenUsers' will allow us to open the remote HKU hive.
+        try:
+            resp = rrp.hOpenUsers(dce)
+        except DCERPCException as e:
+            if 'rpc_s_access_denied' in str(e):
+                logging.debug('Access denied while enumerating Registry Sessions on %s', self.hostname)
+                return []
+            else:
+                logging.debug('Exception connecting to RPC: %s', e)
+        except Exception as e:
+            if str(e).find('Broken pipe') >= 0:
+                return
+            else:
+                raise
+
+        # Once we have a handle on the remote HKU hive, we can call 'BaseRegEnumKey' in a loop in
+        # order to enumerate the subkeys which names are the SIDs of the logged in users.
+        key_handle = resp['phKey']
+        index = 1
+        sid_filter = "^S-1-5-21-[0-9]+-[0-9]+-[0-9]+-[0-9]+$"
+        while True:
+            try:
+                resp = rrp.hBaseRegEnumKey(dce, key_handle, index)
+                sid = resp['lpNameOut'].rstrip('\0')
+                if re.match(sid_filter, sid):
+                    logging.info('User with SID %s is logged in on %s' % (sid, self.hostname))
+                    # Ignore local accounts (best effort, self.sid is only
+                    # populated if we enumerated a group before)
+                    if self.sid and sid.startswith(self.sid):
+                        index += 1
+                        continue
+                    registry_sessions.append({'user': sid})
+                index += 1
+            except:
+                break
+
+        rrp.hBaseRegCloseKey(dce, key_handle)
+        dce.disconnect()
+
+        return registry_sessions
+
     """
     """
     def rpc_get_domain_trusts(self):
