@@ -62,11 +62,11 @@ class ADUtils(object):
         "S-1-5-12": ("Restricted Code", "GROUP"),
         "S-1-5-13": ("Terminal Server Users", "GROUP"),
         "S-1-5-14": ("Remote Interactive Logon", "GROUP"),
-        "S-1-5-15": ("This Organization ", "GROUP"),
-        "S-1-5-17": ("This Organization ", "GROUP"),
+        "S-1-5-15": ("This Organization", "GROUP"),
+        "S-1-5-17": ("IUSR", "USER"),
         "S-1-5-18": ("Local System", "USER"),
         "S-1-5-19": ("NT Authority", "USER"),
-        "S-1-5-20": ("NT Authority", "USER"),
+        "S-1-5-20": ("Network Service", "USER"),
         "S-1-5-80-0": ("All Services ", "GROUP"),
         "S-1-5-32-544": ("Administrators", "GROUP"),
         "S-1-5-32-545": ("Users", "GROUP"),
@@ -95,7 +95,8 @@ class ADUtils(object):
         "S-1-5-32-577": ("RDS Management Servers", "GROUP"),
         "S-1-5-32-578": ("Hyper-V Administrators", "GROUP"),
         "S-1-5-32-579": ("Access Control Assistance Operators", "GROUP"),
-        "S-1-5-32-580": ("Access Control Assistance Operators", "GROUP")
+        "S-1-5-32-580": ("Access Control Assistance Operators", "GROUP"),
+        "S-1-5-32-582": ("Storage Replica Administrators", "GROUP")
     }
 
     FUNCTIONAL_LEVELS = {
@@ -198,9 +199,9 @@ class ADUtils(object):
             if 'ForeignSecurityPrincipals' in dn:
                 resolved['principal'] = domain.upper()
                 resolved['type'] = 'foreignsecurityprincipal'
-                if 'name' in entry['attributes']:
+                ename = ADUtils.get_entry_property(entry, 'name')
+                if ename:
                     # Fix wellknown entries
-                    ename = entry['attributes']['name']
                     if ename in ADUtils.WELLKNOWN_SIDS:
                         name, sidtype = ADUtils.WELLKNOWN_SIDS[ename]
                         resolved['type'] = sidtype.capitalize()
@@ -211,7 +212,7 @@ class ADUtils(object):
                         # Foreign security principal
                         resolved['objectid'] = ename
             else:
-                resolved['type'] = 'Unknown'
+                resolved['type'] = 'Base'
         else:
             accountType = ADUtils.get_entry_property(entry, 'sAMAccountType')
             if accountType in [268435456, 268435457, 536870912, 536870913]:
@@ -245,7 +246,7 @@ class ADUtils(object):
         resolved['principal'] = ('%s@%s' % (account, domain)).upper()
         resolved['type'] = ADUtils.translateSidType(entry['Use']).lower()
 
-        # Computer accoutns have a different type
+        # Computer accounts have a different type
         if resolved['type'] == 'computer':
             short_name = account.rstrip('$')
             resolved['principal'] = ('%s.%s' % (short_name, domain)).upper()
@@ -287,7 +288,7 @@ class ADUtils(object):
         """
         seconds = int(seconds)
         if seconds == 0:
-            return -1
+            return 0
         return int((seconds - 116444736000000000) / 10000000)
 
     @staticmethod
@@ -307,6 +308,17 @@ class ADUtils(object):
         logon_type = res.group(1)
         return (sid, logon_type)
 
+    @staticmethod
+    def ensure_string(data):
+        """
+        Sometimes properties can contain binary data. Since we can't assume encoding, make
+        sure it can be outputted as json
+        """
+        if isinstance(data, bytes):
+            data = repr(data)
+        return data
+
+
 class AceResolver(object):
     """
     This class resolves ACEs containing rights, acetype and a SID to Aces containing
@@ -322,7 +334,6 @@ class AceResolver(object):
         for ace in aces:
             out = {
                 'RightName': ace['rightname'],
-                'AceType': ace['acetype'],
                 'IsInherited': ace['inherited']
             }
             # Is it a well-known sid?
@@ -331,7 +342,7 @@ class AceResolver(object):
                 out['PrincipalType'] = ADUtils.WELLKNOWN_SIDS[ace['sid']][1].capitalize()
             else:
                 try:
-                    entry = self.addomain.sidcache.get(ace['sid'])
+                    linkitem = self.addomain.newsidcache.get(ace['sid'])
                 except KeyError:
                     # Look it up instead
                     # Is this SID part of the current domain? If not, use GC
@@ -339,32 +350,36 @@ class AceResolver(object):
                     ldapentry = self.resolver.resolve_sid(ace['sid'], use_gc)
                     # Couldn't resolve...
                     if not ldapentry:
-                        logging.warning('Could not resolve SID: %s', ace['sid'])
+                        logging.debug('Could not resolve SID: %s', ace['sid'])
                         # Fake it
                         entry = {
-                            'type': 'Unknown',
-                            'principal': ace['sid']
+                            'type': 'Base',
+                            'objectid': ace['sid']
                         }
                     else:
                         entry = ADUtils.resolve_ad_entry(ldapentry)
+                    linkitem = {
+                        "ObjectIdentifier": entry['objectid'],
+                        "ObjectType": entry['type'].capitalize()
+                    }
                     # Entries are cached regardless of validity - unresolvable sids
                     # are not likely to be resolved the second time and this saves traffic
-                    self.addomain.sidcache.put(ace['sid'], entry)
+                    self.addomain.newsidcache.put(ace['sid'], linkitem)
                 out['PrincipalSID'] = ace['sid']
-                out['PrincipalType'] = entry['type']
+                out['PrincipalType'] = linkitem['ObjectType']
             aces_out.append(out)
         return aces_out
 
-    def resolve_binary_sid(self, bsid):
-        sid = LDAP_SID(bsid).formatCanonical()
+    def resolve_sid(self, sid):
+        # Resolve SIDs for SID history purposes
         out = {}
         # Is it a well-known sid?
         if sid in ADUtils.WELLKNOWN_SIDS:
-            out['ObjectID'] = u'%s-%s' % (self.addomain.domain.upper(), sid)
+            out['ObjectIdentifier'] = u'%s-%s' % (self.addomain.domain.upper(), sid)
             out['ObjectType'] = ADUtils.WELLKNOWN_SIDS[sid][1].capitalize()
         else:
             try:
-                entry = self.addomain.sidcache.get(sid)
+                linkitem = self.addomain.newsidcache.get(sid)
             except KeyError:
                 # Look it up instead
                 # Is this SID part of the current domain? If not, use GC
@@ -372,19 +387,23 @@ class AceResolver(object):
                 ldapentry = self.resolver.resolve_sid(sid, use_gc)
                 # Couldn't resolve...
                 if not ldapentry:
-                    logging.warning('Could not resolve SID: %s', sid)
+                    logging.debug('Could not resolve SID: %s', sid)
                     # Fake it
                     entry = {
-                        'type': 'Unknown',
-                        'principal':sid
+                        'type': 'Base',
+                        'objectid':sid
                     }
                 else:
                     entry = ADUtils.resolve_ad_entry(ldapentry)
+                linkitem = {
+                    "ObjectIdentifier": entry['objectid'],
+                    "ObjectType": entry['type'].capitalize()
+                }
                 # Entries are cached regardless of validity - unresolvable sids
                 # are not likely to be resolved the second time and this saves traffic
-                self.addomain.sidcache.put(sid, entry)
-            out['ObjectID'] = sid
-            out['ObjectType'] = entry['type']
+                self.addomain.newsidcache.put(sid, linkitem)
+            out['ObjectIdentifier'] = sid
+            out['ObjectType'] = linkitem['ObjectType']
         return out
 
 class DNSCache(object):
@@ -433,6 +452,10 @@ class SidCache(object):
         with self.lock:
             self._cache[entry] = value
 
+    # Overwrite cache from disk
+    def load(self, cache):
+        self._cache = cache
+
 class SamCache(SidCache):
     """
     Cache for mapping SAM names to principals.
