@@ -25,8 +25,10 @@
 import logging
 import queue
 import threading
+import calendar
 from bloodhound.ad.utils import ADUtils, AceResolver
 from bloodhound.ad.computer import ADComputer
+from bloodhound.ad.structures import LDAP_SID
 from bloodhound.enumeration.acls import AclEnumerator, parse_binary_acl
 from bloodhound.enumeration.outputworker import OutputWorker
 
@@ -53,7 +55,23 @@ class MembershipEnumerator(object):
         """
         Attempt to resolve the membership (DN) of a group to an object
         """
-        # First assume it is a user
+        # New logic here
+        try:
+            linkentry = self.addomain.dncache[member.upper()]
+        except KeyError:
+            use_gc = ADUtils.ldap2domain(member) != self.addomain.domain
+            qobject = self.addomain.objectresolver.resolve_distinguishedname(member, use_gc=use_gc)
+            if qobject is None:
+                return None
+            resolved_entry = ADUtils.resolve_ad_entry(qobject)
+            linkentry = {
+                "ObjectIdentifier": resolved_entry['objectid'],
+                "ObjectType": resolved_entry['type'].capitalize()
+            }
+            self.addomain.dncache[member.upper()] = linkentry
+        return linkentry
+
+        # DEPRECATED
         try:
             resolved_entry = self.addomain.users[member]
         except KeyError:
@@ -81,8 +99,8 @@ class MembershipEnumerator(object):
                     if resolved_entry['type'] == 'Computer':
                         self.addomain.computers[member] = qobject
         return {
-            "MemberId": resolved_entry['objectid'],
-            "MemberType": resolved_entry['type'].capitalize()
+            "ObjectIdentifier": resolved_entry['objectid'],
+            "ObjectType": resolved_entry['type'].capitalize()
         }
 
     @staticmethod
@@ -109,8 +127,6 @@ class MembershipEnumerator(object):
         props['lastlogon'] = ADUtils.win_timestamp_to_unix(
             ADUtils.get_entry_property(entry, 'lastLogon', default=0, raw=True)
         )
-        if props['lastlogon'] == 0:
-            props['lastlogon'] = -1
         props['lastlogontimestamp'] = ADUtils.win_timestamp_to_unix(
             ADUtils.get_entry_property(entry, 'lastlogontimestamp', default=0, raw=True)
         )
@@ -129,14 +145,27 @@ class MembershipEnumerator(object):
         props['title'] = ADUtils.get_entry_property(entry, 'title')
         props['homedirectory'] = ADUtils.get_entry_property(entry, 'homeDirectory')
         props['description'] = ADUtils.get_entry_property(entry, 'description')
-        props['userpassword'] = ADUtils.get_entry_property(entry, 'userPassword')
+        props['userpassword'] = ADUtils.ensure_string(ADUtils.get_entry_property(entry, 'userPassword'))
         props['admincount'] = ADUtils.get_entry_property(entry, 'adminCount', 0) == 1
         if len(ADUtils.get_entry_property(entry, 'msDS-AllowedToDelegateTo', [])) > 0:
             props['allowedtodelegate'] = ADUtils.get_entry_property(entry, 'msDS-AllowedToDelegateTo', [])
-        props['sidhistory'] = ADUtils.get_entry_property(entry, 'sIDHistory', [])
+        props['sidhistory'] = [LDAP_SID(bsid).formatCanonical() for bsid in ADUtils.get_entry_property(entry, 'sIDHistory', [])]
+        # v4 props
+        whencreated = ADUtils.get_entry_property(entry, 'whencreated', default=0)
+        if isinstance(whencreated, int):
+            props['whencreated'] = whencreated
+        else:
+            props['whencreated'] = calendar.timegm(whencreated.timetuple())
+        props['unixpassword'] = ADUtils.ensure_string(ADUtils.get_entry_property(entry, 'unixuserpassword'))
+        props['unicodepassword'] = ADUtils.ensure_string(ADUtils.get_entry_property(entry, 'unicodepwd'))
+        props['logonscript'] = ADUtils.ensure_string(ADUtils.get_entry_property(entry, 'scriptpath'))
+        props['samaccountname'] = ADUtils.ensure_string(ADUtils.get_entry_property(entry, 'sAMAccountName'))
+        # Non-default schema?
+        # props['sfupassword'] = ADUtils.ensure_string(ADUtils.get_entry_property(entry, 'msSFU30Password'))
+        props['sfupassword'] = None
 
-    def enumerate_users(self):
-        filename = 'users.json'
+    def enumerate_users(self, timestamp=""):
+        filename = timestamp + 'users.json'
 
         # Should we include extra properties in the query?
         with_properties = 'objectprops' in self.collect
@@ -163,19 +192,20 @@ class MembershipEnumerator(object):
             user = {
                 "AllowedToDelegate": [],
                 "ObjectIdentifier": ADUtils.get_entry_property(entry, 'objectSid'),
-                "PrimaryGroupSid": MembershipEnumerator.get_primary_membership(entry),
+                "PrimaryGroupSID": MembershipEnumerator.get_primary_membership(entry),
                 "Properties": {
                     "name": resolved_entry['principal'],
                     "domain": self.addomain.domain.upper(),
-                    "objectid": ADUtils.get_entry_property(entry, 'objectSid'),
-                    "distinguishedname":ADUtils.get_entry_property(entry, 'distinguishedName'),
-                    "highvalue": False,
+                    "domainsid": self.addomain.domain_object.sid,
+                    "distinguishedname":ADUtils.get_entry_property(entry, 'distinguishedName').upper(),
                     "unconstraineddelegation": ADUtils.get_entry_property(entry, 'userAccountControl', default=0) & 0x00080000 == 0x00080000,
+                    "trustedtoauth": ADUtils.get_entry_property(entry, 'userAccountControl', default=0) & 0x01000000 == 0x01000000,
                     "passwordnotreqd": ADUtils.get_entry_property(entry, 'userAccountControl', default=0) & 0x00000020 == 0x00000020
                 },
                 "Aces": [],
                 "SPNTargets": [],
-                "HasSIDHistory": []
+                "HasSIDHistory": [],
+                "IsDeleted": ADUtils.get_entry_property(entry, 'isDeleted', default=False)
             }
 
             if with_properties:
@@ -196,7 +226,7 @@ class MembershipEnumerator(object):
                 # Parse SID history
                 if len(user['Properties']['sidhistory']) > 0:
                     for historysid in user['Properties']['sidhistory']:
-                        user['HasSIDHistory'].append(self.aceresolver.resolve_binary_sid(historysid))
+                        user['HasSIDHistory'].append(self.aceresolver.resolve_sid(historysid))
 
             # If this is a GMSA, process it's ACL. We don't bother with threads/processes here
             # since these accounts shouldn't be that common and neither should they have very complex
@@ -204,7 +234,13 @@ class MembershipEnumerator(object):
             if ADUtils.get_entry_property(entry, 'msDS-GroupMSAMembership', default=b'', raw=True) != b'':
                 self.parse_gmsa(user, entry)
 
-            self.addomain.users[entry['dn']] = resolved_entry
+            # Cache link entry for membership resolution
+            linkentry = {
+                "ObjectIdentifier": resolved_entry['objectid'],
+                "ObjectType": resolved_entry['type'].capitalize()
+            }
+            self.addomain.dncache[entry['dn'].upper()] = linkentry
+
             # If we are enumerating ACLs, we break out of the loop here
             # this is because parsing ACLs is computationally heavy and therefor is done in subprocesses
             if acl:
@@ -219,6 +255,8 @@ class MembershipEnumerator(object):
                 # this is solely for consistency with acl parsing, the performance improvement is probably minimal
                 self.result_q.put(user)
 
+        self.write_default_users()
+
         # If we are parsing ACLs, close the parsing pool first
         # then close the result queue and join it
         if acl and not self.disable_pooling:
@@ -231,7 +269,7 @@ class MembershipEnumerator(object):
 
         logging.debug('Finished writing users')
 
-    def enumerate_groups(self):
+    def enumerate_groups(self, timestamp=""):
 
         highvalue = ["S-1-5-32-544", "S-1-5-32-550", "S-1-5-32-549", "S-1-5-32-551", "S-1-5-32-548"]
 
@@ -246,7 +284,7 @@ class MembershipEnumerator(object):
         with_properties = 'objectprops' in self.collect
         acl = 'acl' in self.collect
 
-        filename = 'groups.json'
+        filename = timestamp + 'groups.json'
         entries = self.addc.get_groups(include_properties=with_properties, acl=acl)
 
         logging.debug('Writing groups to file: %s', filename)
@@ -262,7 +300,6 @@ class MembershipEnumerator(object):
 
         for entry in entries:
             resolved_entry = ADUtils.resolve_ad_entry(entry)
-            self.addomain.groups[entry['dn']] = resolved_entry
             try:
                 sid = entry['attributes']['objectSid']
             except KeyError:
@@ -273,27 +310,36 @@ class MembershipEnumerator(object):
                 "ObjectIdentifier": sid,
                 "Properties": {
                     "domain": self.addomain.domain.upper(),
-                    "objectid": sid,
+                    "domainsid": self.addomain.domain_object.sid,
                     "highvalue": is_highvalue(sid),
                     "name": resolved_entry['principal'],
-                    "distinguishedname": ADUtils.get_entry_property(entry, 'distinguishedName')
+                    "distinguishedname": ADUtils.get_entry_property(entry, 'distinguishedName').upper()
                 },
                 "Members": [],
-                "Aces": []
+                "Aces": [],
+                "IsDeleted": ADUtils.get_entry_property(entry, 'isDeleted', default=False)
             }
             if sid in ADUtils.WELLKNOWN_SIDS:
                 # Prefix it with the domain
                 group['ObjectIdentifier'] = '%s-%s' % (self.addomain.domain.upper(), sid)
-                group['Properties']['objectid'] = group['ObjectIdentifier']
             if with_properties:
                 group['Properties']['admincount'] = ADUtils.get_entry_property(entry, 'adminCount', default=0) == 1
                 group['Properties']['description'] = ADUtils.get_entry_property(entry, 'description')
+                whencreated = ADUtils.get_entry_property(entry, 'whencreated', default=0)
+                group['Properties']['whencreated'] = calendar.timegm(whencreated.timetuple())
 
             for member in entry['attributes']['member']:
                 resolved_member = self.get_membership(member)
                 if resolved_member:
                     group['Members'].append(resolved_member)
 
+            # Create cache entry for links
+            link_output = {
+                "ObjectIdentifier": group['ObjectIdentifier'],
+                "ObjectType": 'Group'
+            }
+            self.addomain.dncache[ADUtils.get_entry_property(entry, 'distinguishedName').upper()] = link_output
+
             # If we are enumerating ACLs, we break out of the loop here
             # this is because parsing ACLs is computationally heavy and therefor is done in subprocesses
             if acl:
@@ -322,15 +368,18 @@ class MembershipEnumerator(object):
 
         logging.debug('Finished writing groups')
 
-    def enumerate_computers_dconly(self):
+    def enumerate_computers_dconly(self,timestamp =""):
         '''
         Enumerate computer objects. This function is only used if no
         collection was requested that required connecting to computers anyway.
         '''
-        filename = 'computers.json'
+        filename = timestamp + 'computers.json'
 
+        # Should we include extra properties in the query?
+        with_properties = 'objectprops' in self.collect
         acl = 'acl' in self.collect
-        entries = self.addc.ad.computers
+
+        entries = self.addc.get_computers(include_properties=with_properties, acl=acl)
 
         logging.debug('Writing computers ACL to file: %s', filename)
 
@@ -348,13 +397,11 @@ class MembershipEnumerator(object):
             if not 'attributes' in entry:
                 continue
 
-            if 'dNSHostName' not in entry['attributes']:
-                continue
-
-            hostname = entry['attributes']['dNSHostName']
+            hostname = ADUtils.get_entry_property(entry, 'dNSHostName')
+            samname = ADUtils.get_entry_property(entry, 'sAMAccountName')
             if not hostname:
-                continue
-            samname = entry['attributes']['sAMAccountName']
+                logging.debug('Invalid computer object without hostname: %s', samname)
+                hostname = ''
 
             cobject = ADComputer(hostname=hostname, samname=samname, ad=self.addomain, addc=self.addc, objectsid=entry['attributes']['objectSid'])
             cobject.primarygroup = MembershipEnumerator.get_primary_membership(entry)
@@ -407,6 +454,32 @@ class MembershipEnumerator(object):
         data['Aces'] += self.aceresolver.resolve_aces(aces)
         self.result_q.put(data)
 
+    def write_default_users(self):
+        """
+        Write built-in users to users.json file
+        """
+
+        domainsid = self.addomain.domain_object.sid
+        domainname = self.addomain.domain.upper()
+
+        user = {
+            "AllowedToDelegate": [],
+            "ObjectIdentifier": "%s-S-1-5-20" % domainname,
+            "PrimaryGroupSID": None,
+            "Properties": {
+                "domain": domainname,
+                "domainsid": self.addomain.domain_object.sid,
+                "name": "NT AUTHORITY@%s" % domainname,
+            },
+            "Aces": [],
+            "SPNTargets": [],
+            "HasSIDHistory": [],
+            "IsDeleted": False,
+            "IsACLProtected": False,
+        }
+        self.result_q.put(user)
+
+
     def write_default_groups(self):
         """
         Put default groups in the groups.json file
@@ -417,6 +490,8 @@ class MembershipEnumerator(object):
         entries = self.addc.get_domain_controllers()
 
         group = {
+            "IsDeleted": False,
+            "IsACLProtected": False,
             "ObjectIdentifier": "%s-S-1-5-9" % rootdomain,
             "Properties": {
                 "domain": rootdomain.upper(),
@@ -428,8 +503,8 @@ class MembershipEnumerator(object):
         for entry in entries:
             resolved_entry = ADUtils.resolve_ad_entry(entry)
             memberdata = {
-                "MemberId": resolved_entry['objectid'],
-                "MemberType": resolved_entry['type'].capitalize()
+                "ObjectIdentifier": resolved_entry['objectid'],
+                "ObjectType": resolved_entry['type'].capitalize()
             }
             group["Members"].append(memberdata)
         self.result_q.put(group)
@@ -439,55 +514,58 @@ class MembershipEnumerator(object):
 
         # Everyone
         evgroup = {
+            "IsDeleted": False,
+            "IsACLProtected": False,
             "ObjectIdentifier": "%s-S-1-1-0" % domainname,
             "Properties": {
                 "domain": domainname,
+                "domainsid": self.addomain.domain_object.sid,
                 "name": "EVERYONE@%s" % domainname,
             },
-            "Members": [
-                {
-                    "MemberId": "%s-515" % domainsid,
-                    "MemberType": "Group"
-                },
-                {
-                    "MemberId": "%s-513" % domainsid,
-                    "MemberType": "Group"
-                }
-            ],
+            "Members": [],
             "Aces": []
         }
         self.result_q.put(evgroup)
 
         # Authenticated users
         augroup = {
+            "IsDeleted": False,
+            "IsACLProtected": False,
             "ObjectIdentifier": "%s-S-1-5-11" % domainname,
             "Properties": {
                 "domain": domainname,
+                "domainsid": self.addomain.domain_object.sid,
                 "name": "AUTHENTICATED USERS@%s" % domainname,
             },
-            "Members": [
-                {
-                    "MemberId": "%s-515" % domainsid,
-                    "MemberType": "Group"
-                },
-                {
-                    "MemberId": "%s-513" % domainsid,
-                    "MemberType": "Group"
-                }
-            ],
+            "Members": [],
             "Aces": []
         }
         self.result_q.put(augroup)
 
+        # Interactive
+        iugroup = {
+            "IsDeleted": False,
+            "IsACLProtected": False,
+            "ObjectIdentifier": "%s-S-1-5-4" % domainname,
+            "Properties": {
+                "domain": domainname,
+                "domainsid": self.addomain.domain_object.sid,
+                "name": "INTERACTIVE@%s" % domainname,
+            },
+            "Members": [],
+            "Aces": []
+        }
+        self.result_q.put(iugroup)
 
-    def enumerate_memberships(self):
+
+    def enumerate_memberships(self, timestamp=""):
         """
         Run appropriate enumeration tasks
         """
-        self.enumerate_users()
-        self.enumerate_groups()
+        self.enumerate_users(timestamp)
+        self.enumerate_groups(timestamp)
         if not ('localadmin' in self.collect
                 or 'session' in self.collect
                 or 'loggedon' in self.collect
                 or 'experimental' in self.collect):
-            self.enumerate_computers_dconly()
+            self.enumerate_computers_dconly(timestamp)
